import {v4 as uuidv4} from 'uuid';
import {ESPSDK} from 'lc-esp-sdk-js';

export var ElementsSDK = {};
/*** ELEMENTS CODE ***/

/**
 * Create a new Elements network client. Multiple clients may co-exist within the same browser or window
 * instance.
 *
 * @constructor
 */
ElementsSDK.ElementsClient = function () {
    this.principal = new ESPSDK.EspPrincipal();
}

/**
 * Return a <code>Principal</code> object for the currently logged in end-user.
 * @returns {*}
 */
ElementsSDK.ElementsClient.prototype.getPrincipal = function () {
    return this.principal;
}

/**
 * Attempt to asynchronously open a well-known screen by name. If the operation is successful, <code>okCallback()</code>
 * will be called, and a <code>Screen</code> object will be passed. If the operation fails, <code>errorCallback()</code>
 * will be called instead.
 *
 * @param {string} screenName The name of the screen.
 * @param {function} okCallback Function to call when the screen has been opened.
 * @param {function} errorCallback Function to call if there is an error opening the screen.
 */
ElementsSDK.ElementsClient.prototype.openScreen = function (screenName, okCallback, errorCallback) {
    let sg = {}
    mockInit(sg);
    sg.label = 'Mock Screen';
    sg._t = 'leigh.elements.v16.Screen';
    sg.content = [];

    let panel1 = {};
    mockInit(panel1);
    panel1._t = 'leigh.elements.v16.Panel';
    panel1.label = 'Users';
    panel1.content = [];
    sg.content.push(panel1);

    let table = {};
    mockInit(table);
    table._t = 'leigh.elements.v16.Table';
    table.isReadOnly = false;
    table.columns = [];
    panel1.content.push(table);

    let col1 = {};
    mockInit(col1);
    col1._t = 'leigh.elements.v16.TableColumnCfg';
    col1.label = 'First Name';
    col1.width = 16;
    col1.alignment = 'left';
    col1.isReadOnly = false;
    table.columns.push(col1);

    let col2 = {};
    mockInit(col2);
    col2._t = 'leigh.elements.v16.TableColumnCfg';
    col2.uiIndex = 1;
    col2.label = 'First Name';
    col2.width = 16;
    col2.alignment = 'left';
    col2.isReadOnly = false;
    table.columns.push(col2);

    let panel2 = {};
    mockInit(panel2);
    panel2._t = 'leigh.elements.v16.Panel';
    panel2.uiIndex = 1;
    panel2.label = 'Detail';

    let formLayout = {};
    mockInit(formLayout);
    formLayout._t = 'leigh.elements.v16.FormLayout';
    formLayout.content = [];

    let field1 = {};
    mockInit(field1);
    this.field1id = field1.id;
    field1._t = 'leigh.elements.v16.TextField';
    field1.label = 'First Name';
    field1.isReadOnly = false;
    formLayout.content.push(field1);

    let field2 = {};
    mockInit(field2);
    this.field2id = field2.id;
    field2._t = 'leigh.elements.v16.TextField';
    field2.uiIndex = 1;
    field2.label = 'Last Name';
    field2.isReadOnly = false;
    formLayout.content.push(field2);

    okCallback(new ElementsSDK.ElementsScreen(sg));
}

/**
 * Create a new Elements with the given screen configuration.
 *
 * @param graph
 * @constructor
 */
ElementsSDK.ElementsScreen = function (graph) {
    this._graph = graph;
}

/**
 * Return the screen graph for the currently loaded screen.
 *
 * @returns {*} The screen graph.
 */
ElementsSDK.ElementsScreen.prototype.getScreenGraph = function () {
    return this._graph;
}

/**
 * Close the currently open screen. This will free any related resources on the backend. Further calls to this
 * screen will fail. After the screen has been closed, <code>screenOpen()</code> may be called to open a new
 * screen.
 */
ElementsSDK.ElementsScreen.prototype.close = function () {
    this._screenDef = null;
}

/**
 * Attempt to asynchronously determine the size of the table in rows. On success <code>okCallback(rows)</code>
 * will be called.
 *
 * @parm id The ID of the table.
 * @param okCallback The callback when reading the table has completed.
 * @param errorCallback The callback when an error has occurred.
 */
ElementsSDK.ElementsScreen.prototype.getDatasourceSize = function (id, okCallback, errorCallback) {
    okCallback(10000);
}

/**
 * Attempt to asynchronously read from the table, beginning with <code>startRow</code> and ending
 * with <code>endRow</code>, inclusive. Counting begins with 0. If the table ends before <code>endRow</code>,
 * fewer rows will be returned. On success <code>okCallback()</code> will be called with an array of the
 * row data.
 *
 * @param id The datasource ID.
 * @param startRowIdx The first index to read.
 * @param endRowIdx The last index to read.
 * @param okCallback The callback when reading the table has completed.
 * @param errorCallback The callback when an error has occurred.
 */
ElementsSDK.ElementsScreen.prototype.getDatasourceRows = function (id, startRowIdx, endRowIdx, okCallback, errorCallback) {
    let result = [];

    // In case they read over
    if (endRowIdx > 9999) endRowIdx = 9999;

    for (let i = startRowIdx; i <= endRowIdx; i++) {
        let row = {};
        row[_this.field1id] = 'First' + uuidv4();
        row[_this.field2id] = 'Last' + uuidv4();
        result.push(row);
    }

    okCallback(result);
}

/**
 * Register an update callback handler for this screen. Only one update callback can be registered at a time. When
 * the screen graph is asynchronously updated by the server, this callback will be called and will be passed an array
 * of the update which have occurred. After this call has completed successfully, <code>getScreenGraph()</code> will
 * return the updated graph with the values already applied.
 *
 * After all the updates for a transaction have occurred, the update commit handler will be called: See
 * <code>registerUpdateCommitCallback()</code>.
 *
 * @param updateCallback The update callback.
 */
ElementsSDK.ElementsScreen.prototype.setScreenGraphUpdateCallback = function (updateCallback) {
    // NOP, we don't ever call it in this mock
}

/**
 * Register an update commit callback. During the update process, the library will call
 * <code>registerUpdateCallback()</code> zero or more times. When it is done calling that function
 * for this update cycle, the update commit callback will be called once.
 *
 * This optional functionality allows the user-interface code to buffer changes until they have all
 * been received.
 *
 * @param updateCallback The update callback.
 */
ElementsSDK.ElementsScreen.prototype.setUpdateCommitCallback = function (updateCommitCallback) {
    // NOP, we don't ever call it in this mock
}

/**
 * Register a message callback. If the back-end sends a message which should be displayed to the user
 * in a dialog box, this callback will be called. The be sent a JSON object containing
 * information about the message which is to be displayed to the user.
 *
 * @param messageCallback
 */
ElementsSDK.ElementsScreen.prototype.setMessageCallback = function (messageCallback) {
    // NOP
}

/**
 * This callback will be called if state in a datasource changes. If this is the case,
 * the user-interface should drop any/all cache and re-fetch visible rows on the datasource using the
 * other functions.
 *
 * A JSON object will be passed to the callback with information detailing the changes to the datasource.
 *
 * @param updateDatasourceCallback
 */
ElementsSDK.ElementsScreen.prototype.setDatasourceUpdateCallback = function (updateDatasourceCallback) {

}

/**
 * Update the given screen graph value, typically as the result of a user interaction. This will
 * send the update to the server which will perform any required processing, and, if needed, will send further
 * instructions to the client, at which point the screen graph and/or table data will be updated and the update
 * callback previously registered with <code>registerUpdateCallback()</code> will be called.
 *
 * For input types that the user has interacted with (for example, a text field), the input should retain the user
 * value until otherwise advised. During user editing, this function should be called even if the user input
 * has not defocused the field or hit enter, if the user has not entered a value for more than 2 seconds.
 *
 * After this function is called, commitPendingUpdates() must be called to actually send the update to the backend.
 * Updates may be batched.
 *
 * @param id
 * @param name
 * @param value
 */
ElementsSDK.ElementsScreen.prototype.updateScreenGraphValue = function (id, name, value) {
    console.log('Value changing: ' + id + ' ' + name);
    okCallback();
}

/**
 * Attempt to asynchronously update the provided value, typically as the result of a user interaction. This will
 * send the update to the server which will perform the required processing, and, if needed, will asynchronously
 * send further instructions to the client.
 *
 * After this function is called, commitPendingUpdates() must be called to actually send the update to the backend.
 * Updates may be batched.
 *
 * @param id string Datasource ID
 * @param rowIdx int  RowIDX
 * @param name string Column ID
 * @param name string Value
 *
 */
ElementsSDK.ElementsScreen.prototype.updateDatasourceValue = function (id, rowIdx, name, value) {

}

/**
 * Set the sort order for the given datasource column.
 * After this function is called, commitPendingUpdates() must be called to actually send the update to the backend.
 * Updates may be batched.
 */
ElementsSDK.ElementsScreen.prototype.updateDatasourceColSort = function (datasourceId, columnId, sortOrder) {
    // NOP
}

/**
 * Set the quick filter for the given datasource column.
 *
 * After this function is called, commitPendingUpdates() must be called to actually send the update to the backend.
 * Updates may be batched.
 */
ElementsSDK.ElementsScreen.prototype.updateDatasourceColFilter = function (datasourceId, columnId, sortOrder) {
    // NOP
}

/**
 * Commit any pending updates.
 */
ElementsSDK.ElementsScreen.prototype.commitPendingUpdates = function (okCallback, errorCallback) {

}