package lc.esp.sdk;

import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;

import javax.jms.*;
import java.util.UUID;

import static java.lang.Thread.sleep;

/**
 * This class implements a message consumer which synchronously receives messages from an ESP/ENGINE over the
 * OpenWire protocol.
 *
 * The consumer supports a consumerId. This uniquely identifies the consumer in such a manner that all consumers
 * with the same consumer ID will compete for messages sent to a topic such that only one of the consumers
 * will receive any given messages. See ATG-51 for more information.
 *
 * @author Alex Leigh
 * @since 18.0 (GIPSY DANGER)
 */
public class ESPConsumer implements AutoCloseable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(ESPConsumer.class);
    private final MessageConsumer consumer;
    private final String consumerId;

    public ESPConsumer(ESPAddress destination, Session session, String consumerId) throws JMSException {
        Destination d = destination.toConsumerDestination(session, consumerId);
        logger.debug("Created consumer. [destination: {}]", d);
        consumer = session.createConsumer(d);
        this.consumerId = consumerId;
    }

    public ESPConsumer(ESPAddress destination, Session session) throws JMSException {
       this(destination, session, UUID.randomUUID().toString());
    }

    public ESPMessage receive() throws InterruptedException {
        while (true) {
            try {
                Message msg = consumer.receive();
                logger.debug("Received message: {}", msg);
                return new ESPMessage(msg);
            } catch (Exception e) {
                // TODO: Demote to debug when we are ready
                logger.info("Received invalid message: {}", e);
                sleep(100);
            }
        }
    }

    @Override
    public void close() throws Exception {
        consumer.close();
    }
}
