package lc.esp.sdk;

import lc.mecha.lang.FutureResult;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;

import javax.jms.*;
import java.util.HashMap;
import java.util.UUID;

/**
 * This class implements a requestor which is capable of sending messages to JMS and waiting for a response.
 *
 * @author Alex Leigh
 * @since 18.1
 */
public class ESPRequestor extends BasicallyDangerous implements AutoCloseable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(ESPRequestor.class);
    private final MessageProducer producer;
    private final MessageConsumer consumer;
    private final Session session;
    private final ESPAddress srcAddress;
    // FIXME: This will leak futures. We need a timeout map.
    private final HashMap<String, FutureResult<ESPMessage>> futures = new HashMap<>();

    protected ESPRequestor(Session session) throws JMSException {
        this.session = session;
        srcAddress = new ESPAddress("lc", "client", "session",
                UUID.randomUUID().toString(), ESPAddressClass.QUEUE, ESPMessageClass.COMMAND);
        producer = session.createProducer(null);
        consumer = session.createConsumer(srcAddress.toDestination(session));
    }

    /**
     * Asynchronously send a message to ESP. A {@link FutureResult} will be returned. If a reply to this message
     * is received by the client the FutureResult will be fulfilled.
     */
    public FutureResult<ESPMessage> request(ESPAddress destination, ESPMessage espMessage)
            throws JMSException {

        FutureResult<ESPMessage> future = new FutureResult<>();
        String correlationId = UUID.randomUUID().toString();
        espMessage.setCorrelationId(correlationId);
        espMessage.setReplyTo(srcAddress);
        Message jms = espMessage.toMessage(session);
        futures.put(correlationId, future);

        synchronized (this) {
            producer.send(destination.toDestination(session), jms);
        }

        return future;
    }

    @Override
    public void close() {
        try {
            producer.close();
        } catch (Exception ignored) {
            // NOP
        }

        try {
            consumer.close();
        } catch (Exception ignored) {
            // NOP
        }
    }

    @Override
    public void runDangerously() throws Exception {
        //noinspection InfiniteLoopStatement
        while (true) {
            Message msg = consumer.receive();
            String correlationId = msg.getJMSCorrelationID();
            if (correlationId == null) {
                logger.warn("Received message with no correlation ID. {}", msg);
                break;
            }

            FutureResult<ESPMessage> future = futures.get(correlationId);
            if (future == null) {
                logger.warn("Received message, but no future. [correlationId: {}]", correlationId);
                break;
            }

            future.set(new ESPMessage(msg));
        }
    }
}
